@@ -1,4 +1,4 @@
-/* $Id: jpilot-dump.c,v 1.37 2010-03-03 12:16:27 rousseau Exp $ */
+/* $Id: jpilot-dump.c,v 1.44 2011-02-09 21:37:28 rousseau Exp $ */
 
 /*******************************************************************************
  * jpilot-dump.c
@@ -72,39 +72,38 @@ char *formatM;
 char *formatA;
 char *formatT;
 
-/* Hacks: Various routines we call from the rest of the jpilot codebase 
- *        expect a graphical environment.  In order for the code to link
- *        properly we need to define various things that are never used. */
+/* Start Hack */
+/* FIXME: The following is a hack.
+ * The variables below are global variables in jpilot.c which are unused in
+ * this code but must be instantiated for the code to compile.  
+ * The same is true of the functions which are only used in GUI mode. */
 pid_t jpilot_master_pid = -1;
+int pipe_to_parent;
 GtkWidget *glob_dialog;
 GtkWidget *glob_date_label;
-int pipe_to_parent;
-/* End hacks */
+gint glob_date_timer_tag;
+
+void output_to_pane(const char *str) { return; }
+int sync_once(struct my_sync_info *sync_info) { return EXIT_SUCCESS; }
+/* End Hack */
 
 /****************************** Main Code *************************************/
 static void fprint_jpd_usage_string(FILE *out)
 {
-   fprintf(out, "%s-dump [ +format [-v] || [-h] || [-f] || [-D] || [-i] || [-A] || [-T] || [-M] || [-N] ]\n", EPN);
-   fprintf(out, "%s", _(" +D +A +T +M format like date +format.\n"));
-   fprintf(out, "%s", _(" -v displays version and exits.\n"));
-   fprintf(out, "%s", _(" -h displays help and exits.\n"));
-   fprintf(out, "%s", _(" -f displays help for format codes.\n"));
-   fprintf(out, "%s", _(" -D dump DateBook.\n"));
-   fprintf(out, "%s", _(" -i dump DateBook in iCalendar format.\n"));
-   fprintf(out, "%s", _(" -N dump appts for today in DateBook.\n"));
-   fprintf(out, "%s", _(" -NYYYY/MM/DD dump appts on YYYY/MM/DD in DateBook.\n"));
-   fprintf(out, "%s", _(" -A dump Address book.\n"));
-   fprintf(out, "%s", _(" -T dump ToDo list as CSV.\n"));
-   fprintf(out, "%s", _(" -M dump Memos.\n"));
+   fprintf(out, "%s-dump [ +format [-v] || [-h] || [-f] || [-D] || [-i] || [-N] || [-A] || [-T] || [-M] ]\n", EPN);
+   fprintf(out, _(" +D +A +T +M format like date +format.\n"));
+   fprintf(out, _(" -v display version and exit\n"));
+   fprintf(out, _(" -h display help text\n"));
+   fprintf(out, _(" -f display help for format codes\n"));
+   fprintf(out, _(" -D dump DateBook\n"));
+   fprintf(out, _(" -i dump DateBook in iCalendar format\n"));
+   fprintf(out, _(" -N dump appts for today in DateBook\n"));
+   fprintf(out, _(" -NYYYY/MM/DD dump appts on YYYY/MM/DD in DateBook\n"));
+   fprintf(out, _(" -A dump Address book\n"));
+   fprintf(out, _(" -T dump ToDo list as CSV\n"));
+   fprintf(out, _(" -M dump Memos\n"));
 }
 
-/* Hacks: jpilot-dump is a command-line tool and not all subroutines need
- *        coding. */
-void output_to_pane(const char *str);
-void output_to_pane(const char *str) {}
-int sync_once(struct my_sync_info *sync_info) { return EXIT_SUCCESS; }
-/* End hacks */
-
 /* convert from UTF8 to local encoding */
 static void utf8_to_local(char *str)
 {
@@ -170,12 +169,12 @@ static int dumpical(void)
 
    get_pref(PREF_USER, NULL, &svalue);
    g_strlcpy(text, svalue, 128);
+   text[127] = '\0';
    charset_p2j(text, 128, char_set);
    str_to_ical_str(username, sizeof(username), text);
-   get_pref(PREF_USER_ID, &userid, &svalue);
+   get_pref(PREF_USER_ID, &userid, NULL);
    gethostname(text, sizeof(hostname));
    text[sizeof(hostname)-1]='\0';
-   charset_p2j(text, sizeof(hostname), char_set);
    str_to_ical_str(hostname, sizeof(hostname), text);
    time(&ltime);
    now = gmtime(&ltime);
@@ -331,7 +330,7 @@ static int dumpical(void)
             memset(&ical_time, 0, sizeof(ical_time));
             ical_time.tm_year = mappt->appt.repeatEnd.tm_year;
             ical_time.tm_mon  = mappt->appt.repeatEnd.tm_mon;
-            ical_time.tm_mday = mappt->appt.repeatEnd.tm_mday + 1;
+            ical_time.tm_mday = mappt->appt.repeatEnd.tm_mday;
             ical_time.tm_isdst= -1;
             mktime(&ical_time);
             printf(";UNTIL=%04d%02d%02d",
@@ -652,10 +651,8 @@ static int dumpaddress(void)
    AddressList *tal, *al;
    int num, i;
    struct AddressAppInfo ai;
-   static struct AddressAppInfo *glob_PAddress_app_info;
 
    get_address_app_info(&ai);
-   glob_PAddress_app_info = &ai;
 
    al = NULL;
    i = 0;
@@ -691,7 +688,7 @@ static int dumpaddress(void)
                      i++;
                      break;
                 case 'C' :
-                     printf("%s", glob_PAddress_app_info->category.name[tal->maddr.attrib & 0x0F]);
+                     printf("%s", ai.category.name[tal->maddr.attrib & 0x0F]);
                      i++;
                      break;
                 case 'N' :   /* normal output */
@@ -776,10 +773,8 @@ static int dumptodo(void)
    int num, i;
    int year, month, day, hour, minute;
    struct ToDoAppInfo ai;
-   static struct ToDoAppInfo *glob_Ptodo_app_info;
 
    get_todo_app_info(&ai);
-   glob_Ptodo_app_info = &ai;
 
    al = NULL;
    num = get_todos(&al, SORT_ASCENDING);
@@ -827,7 +822,7 @@ static int dumptodo(void)
                      i++;
                      break;
                 case 'C' :
-                     printf("%s", glob_Ptodo_app_info->category.name[tal->mtodo.attrib & 0x0F]);
+                     printf("%s", ai.category.name[tal->mtodo.attrib & 0x0F]);
                      i++;
                      break;
                 case 'A' :
@@ -1038,10 +1033,8 @@ static int dumpmemo(void)
    MemoList *tal, *al;
    int num,i;
    struct MemoAppInfo ai;
-   static struct MemoAppInfo *glob_PMemo_app_info;
 
    get_memo_app_info(&ai);
-   glob_PMemo_app_info = &ai;
 
    al = NULL;
    i = 0;
@@ -1085,7 +1078,7 @@ static int dumpmemo(void)
                      i++;
                      break;
                 case 'C' :
-                     printf("%s", glob_PMemo_app_info->category.name[tal->mmemo.attrib & 0x0F]);
+                     printf("%s", ai.category.name[tal->mmemo.attrib & 0x0F]);
                      i++;
                      break;
                 case 'N' :   /* normal output */
