@@ -1,4 +1,4 @@
-/* $Id: libplugin.h,v 1.30 2010-03-03 12:02:40 rousseau Exp $ */
+/* $Id: libplugin.h,v 1.37 2011-02-09 20:36:45 rousseau Exp $ */
 
 /*******************************************************************************
  * libplugin.h
@@ -27,6 +27,7 @@
 #include <gtk/gtk.h>
 #include <time.h>
 #include <pi-appinfo.h>
+#include "log.h"
 
 /*
  * PLUGIN API for J-Pilot
@@ -53,9 +54,9 @@
 
 
 typedef struct {
-  unsigned char Offset[4];  /*4 bytes offset from BOF to record */
-  unsigned char attrib;
-  unsigned char unique_ID[3];
+   unsigned char Offset[4];  /*4 bytes offset from BOF to record */
+   unsigned char attrib;
+   unsigned char unique_ID[3];
 } record_header;
 
 typedef struct {
@@ -144,12 +145,11 @@ int get_next_unique_pc_id(unsigned int *next_unique_id);
 #define DIALOG_SAID_1        454
 #define DIALOG_SAID_PRINT    454
 #define DIALOG_SAID_FOURTH   454
-#define DIALOG_SAID_CURRENT  454
 #define DIALOG_SAID_2        455
 #define DIALOG_SAID_LAST     455
-#define DIALOG_SAID_ALL      455
 #define DIALOG_SAID_3        456
 #define DIALOG_SAID_CANCEL   456
+#define DIALOG_SAID_4        457
 
 #define JP_LOG_DEBUG  1    /*debugging info for programmers, and bug reports */
 #define JP_LOG_INFO   2    /*info, and misc messages */
@@ -161,8 +161,6 @@ int get_next_unique_pc_id(unsigned int *next_unique_id);
 
 #define JPILOT_EOF -7
 
-extern int jp_logf(int level, const char *format, ...);
-
 /* This bit means that this record is of no importance anymore */
 #define SPENT_PC_RECORD_BIT 256
 
@@ -201,13 +199,16 @@ struct search_result
 
 
 void plugin_version(int *major_version, int *minor_version);
-int plugin_get_help_name(char *name, int len);
 int plugin_get_name(char *name, int len);
 int plugin_get_menu_name(char *name, int len);
+int plugin_get_help_name(char *name, int len);
 int plugin_get_db_name(char *db_name, int len);
 int plugin_startup(jp_startup_info *info);
 int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id);
 int plugin_help(char **text, int *width, int *height);
+int plugin_print(void);
+int plugin_import(GtkWidget *window);
+int plugin_export(GtkWidget *window);
 int plugin_gui_cleanup(void);
 int plugin_pre_sync_pre_connect(void);
 int plugin_pre_sync(void);
@@ -216,26 +217,25 @@ int plugin_search(const char *search_string, int case_sense, struct search_resul
 int plugin_post_sync(void);
 int plugin_exit_cleanup(void);
 int plugin_unpack_cai_from_ai(struct CategoryAppInfo *cai,
-			      unsigned char *ai_raw, int len);
+                              unsigned char *ai_raw, int len);
 int plugin_pack_cai_into_ai(struct CategoryAppInfo *cai,
-			    unsigned char *ai_raw, int len);
+                            unsigned char *ai_raw, int len);
 /* callbacks are needed for print */
 
 void jp_init(void);
-extern FILE *jp_open_home_file(char *filename, char *mode);
 
 /* This takes the value of $JPILOT_HOME and appends /.jpilot/ and {file}
  * onto it and puts it into full_name.  max_size is the size if the
  * supplied buffer full_name
  */
-int jp_get_home_file_name(char *file, char *full_name, int max_size);
+int jp_get_home_file_name(const char *file, char *full_name, int max_size);
 
 /*
  * DB_name should be without filename ext, e.g. MemoDB
  * bufp is the packed app info block
  * size_in is the size of bufp
  */
-int jp_pdb_file_write_app_block(char *DB_name, void *bufp, int size_in);
+int jp_pdb_file_write_app_block(const char *DB_name, void *bufp, int size_in);
 
 /*
  * widget is a widget inside the main window used to get main window handle
@@ -245,13 +245,6 @@ int jp_pdb_file_write_app_block(char *DB_name, void *bufp, int size_in);
  */
 int jp_edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
 
-/*************************************
- * convert char code
- *************************************/
-extern void jp_charset_j2p(char *buf, int max_len);
-extern void jp_charset_p2j(char *buf, int max_len);
-extern char* jp_charset_p2newj(const char *buf, int max_len);
-
 /* file must not be open elsewhere when this is called, the first line is 0 */
 int jp_install_remove_line(int deleted_line);
 
@@ -260,27 +253,27 @@ int jp_install_append_line(char *line);
 /*
  * Get the application info block
  */
-int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size);
+int jp_get_app_info(const char *DB_name, unsigned char **buf, int *buf_size);
 /*
  * Read a pdb file out of the $(JPILOT_HOME || HOME)/.jpilot/ directory
  * It also reads the PC file
  */
-int jp_read_DB_files(char *DB_name, GList **records);
+int jp_read_DB_files(const char *DB_name, GList **records);
 
 /*
  *This deletes a record from the appropriate Datafile
  */
-int jp_delete_record(char *DB_name, buf_rec *br, int flag);
+int jp_delete_record(const char *DB_name, buf_rec *br, int flag);
 /*
  *This undeletes a record from the appropriate Datafile
  */
-int jp_undelete_record(char *DB_name, buf_rec *br, int flag);
+int jp_undelete_record(const char *DB_name, buf_rec *br, int flag);
 /*
  * Free the record list
  */
 int jp_free_DB_records(GList **records);
 
-int jp_pc_write(char *DB_name, buf_rec *br);
+int jp_pc_write(const char *DB_name, buf_rec *br);
 
 const char *jp_strstr(const char *haystack, const char *needle, int case_sense);
 
@@ -300,13 +293,6 @@ int unlink_file(char *filename);
 /* */
 int get_app_info_size(FILE *in, int *size);
 
-/*
- * Widget must be some widget used to get the main window from.
- * The main window passed in would be fastest.
- * changed is MODIFY_FLAG, or NEW_FLAG
- */
-int dialog_save_changed_record(GtkWidget *widget, int changed);
-
 /* mon 0-11
  * day 1-31
  * year (year - 1900)
@@ -314,8 +300,8 @@ int dialog_save_changed_record(GtkWidget *widget, int changed);
  * After a new date is selected it will return mon, day, year
  */
 int jp_cal_dialog(GtkWindow *main_window,
-		  const char *title, int monday_is_fdow,
-		  int *mon, int *day, int *year);
+                  const char *title, int monday_is_fdow,
+                  int *mon, int *day, int *year);
 
 /*
  * The preferences interface makes it easy to read and write name/value pairs
