@@ -1,4 +1,4 @@
-/* $Id: prefs.c,v 1.88 2010-03-29 05:44:30 rikster5 Exp $ */
+/* $Id: prefs.c,v 1.90 2011-02-10 23:28:07 rikster5 Exp $ */
 
 /*******************************************************************************
  * prefs.c
@@ -46,7 +46,7 @@
 #define NUM_PAPER_SIZES 2
 
 /******************************* Global vars **********************************/
-int t_fmt_ampm = TRUE;
+static int t_fmt_ampm = TRUE;
 
 /* These are the default settings */
 /* name, usertype, filetype, ivalue, char *svalue, svalue_size; */
@@ -151,6 +151,7 @@ static prefType glob_prefs[NUM_PREFS] = {
    {"expense_sort_column", INTTYPE, INTTYPE, 0, NULL, 0},
    {"expense_sort_order", INTTYPE, INTTYPE, 0, NULL, 0},
    {"keyr_export_filename", CHARTYPE, CHARTYPE, 0, NULL, 0},
+   {"external_editor", CHARTYPE, CHARTYPE, 0, NULL, 0},
 };
 
 struct name_list {
@@ -204,6 +205,10 @@ void pref_init(void)
          glob_prefs[i].svalue=strdup("mozilla-thunderbird -remote \"mailto(%s)\"");
          glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
          break;
+       case PREF_EXTERNAL_EDITOR:
+         glob_prefs[i].svalue=strdup("gvim -f");
+         glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
+         break;
        default:
          glob_prefs[i].svalue=strdup("");
          glob_prefs[i].svalue_size=1;
