@@ -1,4 +1,4 @@
-/* $Id: prefs_gui.c,v 1.73 2010-03-29 05:44:30 rikster5 Exp $ */
+/* $Id: prefs_gui.c,v 1.82 2011-02-10 23:28:07 rikster5 Exp $ */
 
 /*******************************************************************************
  * prefs_gui.c
@@ -38,6 +38,7 @@ static GtkWidget *window;
 static GtkWidget *main_window;
 static GtkWidget *port_entry;
 static GtkWidget *backups_entry;
+static GtkWidget *ext_editor_entry;
 static GtkWidget *alarm_command_entry;
 static GtkWidget *mail_command_entry;
 static GtkWidget *todo_days_due_entry;
@@ -48,9 +49,9 @@ extern GtkTooltips *glob_tooltips;
 extern unsigned char skip_plugins;
 #endif
 
-/* Serial Port Menu */
+/* Sync Port Menu */
 static GtkWidget *port_menu;
-GtkWidget *port_menu_item[10];
+static GtkWidget *port_menu_item[10];
 static char *port_choices[]={
    "other", "usb:",
    "/dev/ttyUSB0", "/dev/ttyUSB1", "/dev/ttyUSB2", "/dev/ttyUSB3",
@@ -58,11 +59,14 @@ static char *port_choices[]={
    NULL
 };
 
+/* Serial Rate Menu */
+static GtkWidget *rate_menu;
+
 /****************************** Main Code *************************************/
 #ifdef COLORS
 
 /* This doesn't work quite right.  There is supposedly no way to do it in GTK. */
-void r(GtkWidget *w, gpointer data)
+static void r(GtkWidget *w, gpointer data)
 {
    GtkStyle *style;
 
@@ -73,7 +77,7 @@ void r(GtkWidget *w, gpointer data)
    }
 }
 
-void set_colors()
+static void set_colors()
 {
    GtkStyle* style;
    int i;
@@ -92,17 +96,24 @@ void set_colors()
 }
 #endif /* #ifdef COLORS */
 
-/* Serial Port menu code */
+/* Sync Port menu code */
 static void cb_serial_port_menu(GtkWidget *widget,
                                 gpointer   data)
 {
    if (!widget)
-     return;
+      return;
    if (!(GTK_CHECK_MENU_ITEM(widget))->active) {
       return;
    }
 
-   gtk_entry_set_text(GTK_ENTRY(port_entry), port_choices[GPOINTER_TO_INT(data)]);
+   char *port_str = port_choices[GPOINTER_TO_INT(data)]; 
+   gtk_entry_set_text(GTK_ENTRY(port_entry), port_str);
+   if (! strcmp(port_str, "usb:")) {
+      gtk_widget_set_sensitive(rate_menu, FALSE);
+   } else {
+      gtk_widget_set_sensitive(rate_menu, TRUE);
+   }
+
 
    return;
 }
@@ -134,7 +145,8 @@ static int make_serial_port_menu(GtkWidget **port_menu)
 
       /* We don't want a callback if "other" is selected */
       if (i) {
-         gtk_signal_connect(GTK_OBJECT(port_menu_item[i]), "activate", GTK_SIGNAL_FUNC(cb_serial_port_menu),
+         gtk_signal_connect(GTK_OBJECT(port_menu_item[i]), "activate", 
+                            GTK_SIGNAL_FUNC(cb_serial_port_menu),
                             GINT_TO_POINTER(i));
       }
 
@@ -147,7 +159,7 @@ static int make_serial_port_menu(GtkWidget **port_menu)
    return EXIT_SUCCESS;
 }
 
-/* End Serial Port Menu code */
+/* End Sync Port Menu code */
 
 static void cb_pref_menu(GtkWidget *widget, gpointer data)
 {
@@ -155,7 +167,7 @@ static void cb_pref_menu(GtkWidget *widget, gpointer data)
    int value;
 
    if (!widget)
-     return;
+      return;
    if (!(GTK_CHECK_MENU_ITEM(widget))->active) {
       return;
    }
@@ -264,11 +276,6 @@ static void cb_checkbox_todo_days_till_due(GtkWidget *widget, gpointer data)
 
 static void cb_checkbox_show_tooltips(GtkWidget *widget, gpointer data)
 {
-   if (GTK_TOGGLE_BUTTON(widget)->active)
-      gtk_tooltips_enable(glob_tooltips);
-   else
-      gtk_tooltips_disable(glob_tooltips);
-
    set_pref(PREF_SHOW_TOOLTIPS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
 
@@ -388,7 +395,7 @@ static void add_checkbutton(const char *text,
 
    /* Set the button state based on option value */
    if (get_pref_int_default(which, 0))
-       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
 
    /* Set button callback */
    gtk_signal_connect(GTK_OBJECT(checkbutton), "clicked", GTK_SIGNAL_FUNC(cb),
@@ -511,7 +518,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
                              1, 2, 0, 1);
 
-   get_pref(PREF_CHAR_SET, &ivalue, &cstr);
+   get_pref(PREF_CHAR_SET, &ivalue, NULL);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Shortdate */
@@ -524,7 +531,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
                              1, 2, 1, 2);
 
-   get_pref(PREF_SHORTDATE, &ivalue, &cstr);
+   get_pref(PREF_SHORTDATE, &ivalue, NULL);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Longdate */
@@ -537,7 +544,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
                              1, 2, 2, 3);
 
-   get_pref(PREF_LONGDATE, &ivalue, &cstr);
+   get_pref(PREF_LONGDATE, &ivalue, NULL);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Time */
@@ -550,7 +557,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
                              1, 2, 3, 4);
 
-   get_pref(PREF_TIME, &ivalue, &cstr);
+   get_pref(PREF_TIME, &ivalue, NULL);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /**********************************************************************/
@@ -570,11 +577,11 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
                              2, 3, 0, 1);
 
-   get_pref(PREF_RCFILE, &ivalue, &cstr);
+   get_pref(PREF_RCFILE, &ivalue, NULL);
    gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
 
    /* Port */
-   label = gtk_label_new(_("Serial Port"));
+   label = gtk_label_new(_("Sync Port"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
                              0, 1, 1, 2);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -583,7 +590,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    entry_set_multiline_truncate(GTK_ENTRY(port_entry), TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(port_entry),
                              2, 3, 1, 2);
-   get_pref(PREF_PORT, &ivalue, &cstr);
+   get_pref(PREF_PORT, NULL, &cstr);
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(port_entry), cstr);
    }
@@ -591,7 +598,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
                       "changed", GTK_SIGNAL_FUNC(cb_text_entry),
                       GINT_TO_POINTER(PREF_PORT));
 
-   /* Serial Port Menu */
+   /* Sync Port Menu */
    /* Note that port_entry must exist before we call this function */
    make_serial_port_menu(&port_menu);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(port_menu),
@@ -603,12 +610,19 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
                              0, 2, 2, 3);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 
-   make_pref_menu(&pref_menu, PREF_RATE);
-   gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(pref_menu),
+   make_pref_menu(&rate_menu, PREF_RATE);
+   gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(rate_menu),
                              2, 3, 2, 3);
 
-   get_pref(PREF_RATE, &ivalue, &cstr);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(pref_menu), ivalue);
+   get_pref(PREF_RATE, &ivalue, NULL);
+   gtk_option_menu_set_history(GTK_OPTION_MENU(rate_menu), ivalue);
+
+   /* Disable Serial Rate menu if sync port is USB */
+   if (! strcmp(cstr, "usb:")) {
+      gtk_widget_set_sensitive(rate_menu, FALSE);
+   } else {
+      gtk_widget_set_sensitive(rate_menu, TRUE);
+   }
 
    /* Number of backups */
    label = gtk_label_new(_("Number of backups to be archived"));
@@ -621,7 +635,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_widget_set_usize(backups_entry, 30, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(backups_entry),
                              2, 3, 3, 4);
-   get_pref(PREF_NUM_BACKUPS, &ivalue, &cstr);
+   get_pref(PREF_NUM_BACKUPS, &ivalue, NULL);
    sprintf(temp_str, "%ld", ivalue);
    gtk_entry_set_text(GTK_ENTRY(backups_entry), temp_str);
    gtk_signal_connect(GTK_OBJECT(backups_entry),
@@ -642,7 +656,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
       PREF_CONFIRM_FILE_INSTALL, vbox_settings, cb_checkbox_set_pref);
 
    /* Show tooltips check box */
-   add_checkbutton(_("Show popup tooltips (default YES)"),
+   add_checkbutton(_("Show popup tooltips (default YES) (requires restart)"),
                    PREF_SHOW_TOOLTIPS, vbox_settings,
                    cb_checkbox_show_tooltips);
 
@@ -748,7 +762,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 
    mail_command_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
 
-   get_pref(PREF_MAIL_COMMAND, &ivalue, &cstr);
+   get_pref(PREF_MAIL_COMMAND, NULL, &cstr);
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(mail_command_entry), cstr);
    }
@@ -825,7 +839,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 
    todo_days_due_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
    entry_set_multiline_truncate(GTK_ENTRY(todo_days_due_entry), TRUE);
-   get_pref(PREF_TODO_DAYS_TILL_DUE, &ivalue, &cstr);
+   get_pref(PREF_TODO_DAYS_TILL_DUE, &ivalue, NULL);
    temp[0]='\0';
    g_snprintf(temp, sizeof(temp), "%ld", ivalue);
       gtk_entry_set_text(GTK_ENTRY(todo_days_due_entry), temp);
@@ -881,6 +895,30 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
       break;
    }
 
+   hseparator = gtk_hseparator_new();
+   gtk_box_pack_start(GTK_BOX(vbox_memo), hseparator, FALSE, FALSE, 3);
+
+   /* External Editor Command to execute */
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox_memo), hbox_temp, FALSE, FALSE, 0);
+
+   label = gtk_label_new(_("External Editor"));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 0);
+
+   ext_editor_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
+   get_pref(PREF_EXTERNAL_EDITOR, NULL, &cstr);
+   if (cstr) {
+      gtk_entry_set_text(GTK_ENTRY(ext_editor_entry), cstr);
+   }
+   gtk_signal_connect(GTK_OBJECT(ext_editor_entry),
+                      "changed", GTK_SIGNAL_FUNC(cb_text_entry),
+                      GINT_TO_POINTER(PREF_EXTERNAL_EDITOR));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), ext_editor_entry, TRUE, TRUE, 1);
+
+   label = gtk_label_new(_("Use Ctrl-E inside a memo to launch external editor for memo text"));
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+   gtk_box_pack_start(GTK_BOX(vbox_memo), label, FALSE, FALSE, 0);
+
    /**********************************************************************/
    /* Alarms preference tab */
 
@@ -905,7 +943,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 10);
 
    alarm_command_entry = gtk_entry_new_with_max_length(MAX_PREF_LEN - 2);
-   get_pref(PREF_ALARM_COMMAND, &ivalue, &cstr);
+   get_pref(PREF_ALARM_COMMAND, NULL, &cstr);
    if (cstr) {
       gtk_entry_set_text(GTK_ENTRY(alarm_command_entry), cstr);
    }
@@ -966,11 +1004,11 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    add_checkbutton(_("Sync Manana"),
                    PREF_SYNC_MANANA, vbox_conduits, cb_checkbox_set_pref);
 #endif
-   get_pref(PREF_CHAR_SET, &ivalue, &cstr);
+   get_pref(PREF_CHAR_SET, &ivalue, NULL);
    if (ivalue == CHAR_SET_JAPANESE || ivalue == CHAR_SET_SJIS_UTF) {
-       /* Show use Japanese Kana extention check box */
-       add_checkbutton(_("Use J-OS (Not Japanese PalmOS:WorkPad/CLIE)"),
-                       PREF_USE_JOS, vbox_settings, cb_checkbox_set_pref);
+      /* Show use Japanese Kana extention check box */
+      add_checkbutton(_("Use J-OS (Not Japanese PalmOS:WorkPad/CLIE)"),
+                      PREF_USE_JOS, vbox_settings, cb_checkbox_set_pref);
    }
 
 #ifdef  ENABLE_PLUGINS
