@@ -1,4 +1,4 @@
-/* $Id: todo_gui.c,v 1.163 2010-03-03 12:50:00 rousseau Exp $ */
+/* $Id: todo_gui.c,v 1.182 2011-04-05 16:45:25 rikster5 Exp $ */
 
 /*******************************************************************************
  * todo_gui.c
@@ -53,6 +53,9 @@
 #define CONNECT_SIGNALS 400
 #define DISCONNECT_SIGNALS 401
 
+/* RFCs use CRLF for Internet newline */
+#define CRLF "\x0D\x0A"
+
 /******************************* Global vars **********************************/
 /* Keeps track of whether code is using ToDo, or Tasks database
  * 0 is ToDo, 1 is Tasks */
@@ -91,8 +94,8 @@ static ToDoList *export_todo_list=NULL;
 
 static struct sorted_cats sort_l[NUM_TODO_CAT_ITEMS];
 
-struct ToDoAppInfo todo_app_info;
-int todo_category=CATEGORY_ALL;
+static struct ToDoAppInfo todo_app_info;
+static int todo_category=CATEGORY_ALL;
 static int clist_col_selected;
 static int clist_row_selected;
 static int record_changed;
@@ -100,28 +103,20 @@ static int record_changed;
 /****************************** Prototypes ************************************/
 void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
                        ToDoList **todo_list, int category, int main);
-int todo_clear_details(void);
-int todo_clist_redraw(void);
-static void connect_changed_signals(int con_or_dis);
+static int todo_clear_details(void);
+static int todo_clist_redraw(void);
 static int todo_find(void);
 static void cb_add_new_record(GtkWidget *widget, gpointer data);
+static void connect_changed_signals(int con_or_dis);
 
 /****************************** Main Code *************************************/
+/* Called once on initialization of GUI */
 static void init(void)
 {
    time_t ltime;
    struct tm *now;
    long ivalue;
-   int i;
 
-   clist_col_selected = 1;
-   clist_row_selected = 0;
-
-   for (i=0; i<NUM_TODO_CAT_ITEMS; i++) {
-      todo_cat_menu_item2[i] = NULL;
-   }
-
-   record_changed = CLEAR_FLAG;
    time(&ltime);
    now = localtime(&ltime);
 
@@ -129,6 +124,11 @@ static void init(void)
 
    get_pref(PREF_TODO_DAYS_TILL_DUE, &ivalue, NULL);
    add_days_to_date(&due_date, ivalue);
+
+   clist_row_selected = 0;
+   clist_col_selected = 1;
+
+   record_changed = CLEAR_FLAG;
 }
 
 static void update_due_button(GtkWidget *button, struct tm *t)
@@ -394,7 +394,8 @@ Description: %s\nNote: %s\n", due, todo->priority, complete,
 /*
  * Start Import Code
  */
-static int cb_todo_import(GtkWidget *parent_window, const char *file_path, int type)
+static int cb_todo_import(GtkWidget *parent_window, 
+                          const char *file_path, int type)
 {
    FILE *in;
    char text[65536];
@@ -609,7 +610,6 @@ int todo_import(GtkWidget *window)
    if (todo_version==1) {
       type_desc[1] = NULL;
       type_int[1] = 0;
-
    } 
    */
 
@@ -625,7 +625,7 @@ int todo_import(GtkWidget *window)
  */
 
 static void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
-                       int type, const char *filename)
+                              int type, const char *filename)
 {
    MyToDo *mtodo;
    GList *list, *temp_list;
@@ -643,6 +643,7 @@ static void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
    char pref_time[40];
    char csv_text[65550];
    char *p;
+   gchar *end;
    char username[256];
    char hostname[256];
    const char *svalue;
@@ -698,14 +699,22 @@ static void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 
    /* Special setup for ICAL export */
    if (type == EXPORT_TYPE_ICALENDAR) {
+      get_pref(PREF_CHAR_SET, &char_set, NULL);
+      if (char_set < CHAR_SET_UTF) {
+         jp_logf(JP_LOG_WARN, _("Host character encoding is not UTF-8 based.\n"
+                                " Exported ical file may not be standards-compliant\n"));
+      }
+
       get_pref(PREF_USER, NULL, &svalue);
+      /* Convert User Name stored in Palm character set */
       g_strlcpy(text, svalue, 128);
+      text[127] = '\0';
+      charset_p2j(text, 128, char_set);
       str_to_ical_str(username, sizeof(username), text);
       get_pref(PREF_USER_ID, &userid, NULL);
-      gethostname(text, sizeof(text));
-      text[sizeof(text)-1]='\0';
+      gethostname(text, sizeof(hostname));
+      text[sizeof(hostname)-1]='\0';
       str_to_ical_str(hostname, sizeof(hostname), text);
-
       time(&ltime);
       now = gmtime(&ltime);
    }
@@ -722,8 +731,8 @@ static void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
       switch (type) {
        case EXPORT_TYPE_CSV:
          utf = charset_p2newj(todo_app_info.category.name[mtodo->attrib & 0x0F], 16, char_set);
-	 str_to_csv_str(csv_text, utf);
-	 fprintf(out, "\"%s\",", csv_text);
+         str_to_csv_str(csv_text, utf);
+         fprintf(out, "\"%s\",", csv_text);
          g_free(utf);
          fprintf(out, "\"%s\",", (mtodo->attrib & dlpRecAttrSecret) ? "1":"0");
          fprintf(out, "\"%s\",", mtodo->todo.indefinite ? "1":"0");
@@ -776,16 +785,17 @@ static void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
          /* RFC 2445: Internet Calendaring and Scheduling Core
           *           Object Specification */
          if (i == 0) {
-            fprintf(out, "BEGIN:VCALENDAR\nVERSION:2.0\n");
-            fprintf(out, "PRODID:%s\n", FPI_STRING);
+            fprintf(out, "BEGIN:VCALENDAR"CRLF);
+            fprintf(out, "VERSION:2.0"CRLF);
+            fprintf(out, "PRODID:%s"CRLF, FPI_STRING);
          }
-         fprintf(out, "BEGIN:VTODO\n");
+         fprintf(out, "BEGIN:VTODO"CRLF);
          if (mtodo->attrib & dlpRecAttrSecret) {
-            fprintf(out, "CLASS:PRIVATE\n");
+            fprintf(out, "CLASS:PRIVATE"CRLF);
          }
-         fprintf(out, "UID:palm-todo-%08x-%08lx-%s@%s\n",
+         fprintf(out, "UID:palm-todo-%08x-%08lx-%s@%s"CRLF,
                  mtodo->unique_id, userid, username, hostname);
-         fprintf(out, "DTSTAMP:%04d%02d%02dT%02d%02d%02dZ\n",
+         fprintf(out, "DTSTAMP:%04d%02d%02dT%02d%02d%02dZ"CRLF,
                  now->tm_year+1900,
                  now->tm_mon+1,
                  now->tm_mday,
@@ -794,33 +804,43 @@ static void cb_todo_export_ok(GtkWidget *export_window, GtkWidget *clist,
                  now->tm_sec);
          str_to_ical_str(text, sizeof(text),
                          todo_app_info.category.name[mtodo->attrib & 0x0F]);
-         fprintf(out, "CATEGORIES:%s\n", text);
-         g_snprintf(str1, sizeof(str1), "%s", mtodo->todo.description);
+         fprintf(out, "CATEGORIES:%s"CRLF, text);
+         if (mtodo->todo.description) {
+            g_strlcpy(str1, mtodo->todo.description, 51);
+            /* truncate the string on a UTF-8 character boundary */
+            if (char_set > CHAR_SET_UTF) {
+               if (!g_utf8_validate(str1, -1, (const gchar **)&end))
+                  *end = 0;
+            }
+         } else {
+            /* Handle pathological case with null description. */
+            str1[0] = '\0';
+         }
          if ((p = strchr(str1, '\n'))) {
             *p = '\0';
          }
          str_to_ical_str(text, sizeof(text), str1);
-         fprintf(out, "SUMMARY:%s%s\n", text,
+         fprintf(out, "SUMMARY:%s%s"CRLF, text,
                  strlen(str1) > 49 ? "..." : "");
          str_to_ical_str(text, sizeof(text), mtodo->todo.description);
          fprintf(out, "DESCRIPTION:%s", text);
          if (mtodo->todo.note && mtodo->todo.note[0]) {
             str_to_ical_str(text, sizeof(text), mtodo->todo.note);
-            fprintf(out, "\\n\n %s\n", text);
+            fprintf(out, "\\n"CRLF" %s"CRLF, text);
          } else {
-            fprintf(out, "\n");
+            fprintf(out, ""CRLF);
          }
-         fprintf(out, "STATUS:%s\n", mtodo->todo.complete ? "COMPLETED" : "NEEDS-ACTION");
-         fprintf(out, "PRIORITY:%d\n", mtodo->todo.priority);
+         fprintf(out, "STATUS:%s"CRLF, mtodo->todo.complete ? "COMPLETED" : "NEEDS-ACTION");
+         fprintf(out, "PRIORITY:%d"CRLF, mtodo->todo.priority);
          if (!mtodo->todo.indefinite) {
-            fprintf(out, "DUE;VALUE=DATE:%04d%02d%02d\n",
+            fprintf(out, "DUE;VALUE=DATE:%04d%02d%02d"CRLF,
                     mtodo->todo.due.tm_year+1900,
                     mtodo->todo.due.tm_mon+1,
                     mtodo->todo.due.tm_mday);
          }
-         fprintf(out, "END:VTODO\n");
+         fprintf(out, "END:VTODO"CRLF);
          if (temp_list->next == NULL) {
-            fprintf(out, "END:VCALENDAR\n");
+            fprintf(out, "END:VCALENDAR"CRLF);
          }
          break;
        default:
@@ -917,7 +937,7 @@ static int find_menu_cat_pos(int cat)
 }
 
 static void cb_delete_todo(GtkWidget *widget,
-                    gpointer   data)
+                           gpointer   data)
 {
    MyToDo *mtodo;
    int flag;
@@ -963,7 +983,7 @@ static void cb_delete_todo(GtkWidget *widget,
 }
 
 static void cb_undelete_todo(GtkWidget *widget,
-                      gpointer   data)
+                             gpointer   data)
 {
    MyToDo *mtodo;
    int flag;
@@ -1130,7 +1150,7 @@ static void cb_check_button_no_due_date(GtkWidget *widget, gpointer data)
    }
 }
 
-int todo_clear_details(void)
+static int todo_clear_details(void)
 {
    time_t ltime;
    struct tm *now;
@@ -1267,7 +1287,7 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
 {
    MyToDo *mtodo;
    struct ToDo new_todo;
-   unsigned char attrib;
+   unsigned char attrib = 0;
    int flag;
    int show_priv;
    unsigned int unique_id;
@@ -1329,10 +1349,13 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
       unique_id=0;
       pc_todo_write(&new_todo, NEW_PC_REC, attrib, &unique_id);
    }
-   todo_clist_redraw();
    free_ToDo(&new_todo);
-   glob_find_id = unique_id;
-   todo_find();
+
+   /* Don't return to modified record if search gui active */
+   if (!glob_find_id) {
+      glob_find_id = unique_id;
+   }
+   todo_clist_redraw();
 
    return;
 }
@@ -1342,6 +1365,9 @@ static void clear_mytodos(MyToDo *mtodo)
 {
    mtodo->unique_id=0;
    mtodo->attrib=mtodo->attrib & 0xF8;
+   mtodo->todo.complete=0;
+   mtodo->todo.priority=1;
+   mtodo->todo.indefinite=1;
    if (mtodo->todo.description) {
       free(mtodo->todo.description);
       mtodo->todo.description=strdup("");
@@ -1358,8 +1384,8 @@ static void clear_mytodos(MyToDo *mtodo)
 
 /* Function is used to sort clist based on the completed checkbox */
 static gint GtkClistCompareCheckbox(GtkCList *clist,
-                             gconstpointer ptr1,
-                             gconstpointer ptr2)
+                                    gconstpointer ptr1,
+                                    gconstpointer ptr2)
 {
    GtkCListRow *row1;
    GtkCListRow *row2;
@@ -1389,8 +1415,8 @@ static gint GtkClistCompareCheckbox(GtkCList *clist,
 
 /* Function is used to sort clist based on the Due Date field */
 static gint GtkClistCompareDates(GtkCList *clist,
-                          gconstpointer ptr1,
-                          gconstpointer ptr2)
+                                 gconstpointer ptr1,
+                                 gconstpointer ptr2)
 {
    GtkCListRow *row1,  *row2;
    MyToDo      *mtodo1,*mtodo2;
@@ -1641,20 +1667,133 @@ static gboolean cb_key_pressed_left_side(GtkWidget   *widget,
 
 static gboolean cb_key_pressed_right_side(GtkWidget   *widget, 
                                           GdkEventKey *event,
-                                          gpointer     next_widget)
+                                          gpointer     data)
 {
    if ((event->keyval == GDK_Return) && (event->state & GDK_SHIFT_MASK)) {
       gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
       /* Call clist_selection to handle any cleanup such as a modified record */
       cb_clist_selection(clist, clist_row_selected, TODO_PRIORITY_COLUMN, 
                          GINT_TO_POINTER(1), NULL);
-      gtk_widget_grab_focus(GTK_WIDGET(next_widget));
+      gtk_widget_grab_focus(GTK_WIDGET(clist));
       return TRUE;
    }
+   /* Call external editor for note text */
+   if (data != NULL &&
+      (event->keyval == GDK_e) && (event->state & GDK_CONTROL_MASK)) {
+      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+
+      /* Get current text and place in temporary file */
+      GtkTextIter start_iter;
+      GtkTextIter end_iter;
+      char *text_out;
+
+      gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(todo_note_buffer),
+                                 &start_iter, &end_iter);
+      text_out = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(todo_note_buffer),
+                                          &start_iter, &end_iter, TRUE);
+
+      
+      char tmp_fname[] = "jpilot.XXXXXX";
+      int tmpfd = mkstemp(tmp_fname);
+      if (tmpfd < 0)
+      {
+         jp_logf(JP_LOG_WARN, _("Could not get temporary file name\n"));
+         if (text_out)
+            free(text_out);
+         return TRUE;
+      }
+
+      FILE *fptr = fdopen(tmpfd, "w");
+      if (!fptr) {
+         jp_logf(JP_LOG_WARN, _("Could not open temporary file for external editor\n"));
+         if (text_out)
+            free(text_out);
+         return TRUE;
+      }
+      fwrite(text_out, strlen(text_out), 1, fptr);
+      fwrite("\n", 1, 1, fptr);
+      fclose(fptr);
+
+      /* Call external editor */
+      char command[1024];
+      const char *ext_editor;
+
+      get_pref(PREF_EXTERNAL_EDITOR, NULL, &ext_editor);
+      if (!ext_editor) {
+         jp_logf(JP_LOG_INFO, "External Editor command empty\n");
+         if (text_out)
+            free(text_out);
+         return TRUE;
+      }
+
+      if ((strlen(ext_editor) + strlen(tmp_fname) + 1) > sizeof(command))
+      {
+         jp_logf(JP_LOG_WARN, _("External editor command too long to execute\n"));
+         if (text_out)
+            free(text_out);
+         return TRUE;
+      }
+      g_snprintf(command, sizeof(command), "%s %s", ext_editor, tmp_fname);
+
+      /* jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, _("executing command = [%s]\n"), command); */
+      int r = system(command);
+      
+      if (!r)
+      {
+         /* Read data back from temporary file into memo */
+         char text_in[0xFFFF];
+         size_t bytes_read;
+
+         fptr = fopen(tmp_fname, "rb");
+         if (!fptr) {
+            jp_logf(JP_LOG_WARN, _("Could not open temporary file from external editor\n"));
+            return TRUE;
+         }
+         bytes_read = fread(text_in, 1, 0xFFFF, fptr);
+         fclose(fptr);
+         unlink(tmp_fname);
+
+         text_in[--bytes_read] = '\0';  /* Strip final newline */
+         /* Only update text if it has changed */
+         if (strcmp(text_out, text_in)) {
+            gtk_text_buffer_set_text(GTK_TEXT_BUFFER(todo_note_buffer),
+                                     text_in, -1);
+         }
+      }
+
+      if (text_out)
+         free(text_out);
+
+      return TRUE;
+   }   /* End of external editor if */
 
    return FALSE;
 }
 
+void todo_clist_clear(GtkCList *clist)
+{
+   GtkStyle *base_style, *row_style, *cell_style; 
+   int i;
+
+   base_style = gtk_widget_get_style(GTK_WIDGET(clist));
+  
+   for (i=0; i<GTK_CLIST(clist)->rows ; i++)
+   {
+      row_style = gtk_clist_get_row_style(GTK_CLIST(clist), i);
+      if (row_style && (row_style != base_style))
+      {
+         g_object_unref(row_style);  
+      }
+      cell_style = gtk_clist_get_cell_style(GTK_CLIST(clist), i, TODO_DATE_COLUMN);
+      if (cell_style && (cell_style != base_style))
+      {
+         g_object_unref(cell_style);  
+      }
+   }
+
+   gtk_clist_clear(GTK_CLIST(clist));
+
+}
 void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
                        ToDoList **todo_list, int category, int main)
 {
@@ -1670,8 +1809,8 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    char str[50];
    char str2[TODO_MAX_COLUMN_LEN+2];
    const char *svalue;
-   long hide_completed;
-   long hide_not_due;
+   long hide_completed, hide_not_due;
+   long show_tooltips;
    int show_priv;
    time_t ltime;
    struct tm *now, *due;
@@ -1693,7 +1832,7 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
       gtk_signal_disconnect_by_func(GTK_OBJECT(clist),
                                     GTK_SIGNAL_FUNC(cb_clist_selection), NULL);
    }
-   gtk_clist_clear(GTK_CLIST(clist));
+   todo_clist_clear(GTK_CLIST(clist));
 #ifdef __APPLE__
    gtk_clist_thaw(GTK_CLIST(clist));
    gtk_widget_hide(clist);
@@ -1721,7 +1860,7 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    entries_shown=0;
    for (temp_todo = *todo_list, i=0; temp_todo; temp_todo=temp_todo->next) {
       if ( ((temp_todo->mtodo.attrib & 0x0F) != category) &&
-          category != CATEGORY_ALL) {
+             category != CATEGORY_ALL) {
          continue;
       }
 
@@ -1789,11 +1928,11 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 
       /* Print the due date */
       if (!temp_todo->mtodo.todo.indefinite) {
-          get_pref(PREF_SHORTDATE, NULL, &svalue);
-          strftime(str, sizeof(str), svalue, &(temp_todo->mtodo.todo.due));
+         get_pref(PREF_SHORTDATE, NULL, &svalue);
+         strftime(str, sizeof(str), svalue, &(temp_todo->mtodo.todo.due));
       }
       else {
-          sprintf(str, _("No date"));
+         sprintf(str, _("No date"));
       }
       gtk_clist_set_text(GTK_CLIST(clist), entries_shown, TODO_DATE_COLUMN, str);
       /* Print the todo text */
@@ -1854,14 +1993,20 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 
    /* If there are items in the list, highlight the selected row */
    if ((main) && (entries_shown>0)) {
-      /* Select the existing requested row, or row 0 if that is impossible */
-      if (clist_row_selected < entries_shown)
+      /* First, select any record being searched for */
+      if (glob_find_id)
+      {
+         todo_find();
+      }
+      /* Second, try the currently selected row */
+      else if (clist_row_selected < entries_shown)
       {
          clist_select_row(GTK_CLIST(clist), clist_row_selected, TODO_PRIORITY_COLUMN);
          if (!gtk_clist_row_is_visible(GTK_CLIST(clist), clist_row_selected)) {
             gtk_clist_moveto(GTK_CLIST(clist), clist_row_selected, 0, 0.5, 0.0);
          }
       }
+      /* Third, select row 0 if nothing else is possible */
       else
       {
          clist_select_row(GTK_CLIST(clist), 0, TODO_PRIORITY_COLUMN);
@@ -1872,12 +2017,13 @@ void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    gtk_clist_thaw(GTK_CLIST(clist));
 
    if (tooltip_widget) {
+      get_pref(PREF_SHOW_TOOLTIPS, &show_tooltips, NULL);
       if (todo_list==NULL) {
-         gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, _("0 records"), NULL);
+         set_tooltip(show_tooltips, glob_tooltips, tooltip_widget, _("0 records"), NULL);
       }
       else {
          sprintf(str, _("%d of %d records"), entries_shown, num_entries);
-         gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, str, NULL);
+         set_tooltip(show_tooltips, glob_tooltips, tooltip_widget, str, NULL);
       }
    }
 
@@ -1938,7 +2084,7 @@ static gboolean cb_key_pressed_shift_tab(GtkWidget *widget,
 }
 
 /* This redraws the clist and goes back to the same line number */
-int todo_clist_redraw(void)
+static int todo_clist_redraw(void)
 {
    todo_update_clist(clist, category_menu1, &glob_todo_list, todo_category, TRUE);
 
@@ -2001,7 +2147,7 @@ int todo_refresh(void)
         (GTK_CHECK_MENU_ITEM(todo_cat_menu_item1[index]), TRUE);
       gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu1), index2);
    }
-   todo_find();
+
    return EXIT_SUCCESS;
 }
 
@@ -2020,10 +2166,12 @@ int todo_gui_cleanup(void)
    set_pref(PREF_LAST_TODO_CATEGORY, todo_category, NULL, TRUE);
    set_pref(PREF_TODO_SORT_COLUMN, clist_col_selected, NULL, TRUE);
    set_pref(PREF_TODO_SORT_ORDER, GTK_CLIST(clist)->sort_type, NULL, TRUE);
+   todo_clist_clear(GTK_CLIST(clist));
 
    return EXIT_SUCCESS;
 }
 
+/* Main function */
 int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
 {
    GtkWidget *scrolled_window;
@@ -2041,19 +2189,20 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    int i;
    GSList *group;
    long ivalue;
-   const char *svalue;
    char *titles[]={"","","","",""};
    GtkAccelGroup *accel_group;
    long char_set;
+   long show_tooltips;
    char *cat_name;
 
    get_pref(PREF_TODO_VERSION, &todo_version, NULL);
 
    init();
 
-   get_pref(PREF_CHAR_SET, &char_set, NULL);
    get_todo_app_info(&todo_app_info);
 
+   /* Initialize categories */
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
    for (i=1; i<NUM_TODO_CAT_ITEMS; i++) {
       cat_name = charset_p2newj(todo_app_info.category.name[i], 31, char_set);
       strcpy(sort_l[i-1].Pcat, cat_name);
@@ -2077,16 +2226,19 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    get_pref(PREF_LAST_TODO_CATEGORY, &ivalue, NULL);
    todo_category = ivalue;
 
-   if ((todo_category != CATEGORY_ALL) && (todo_app_info.category.name[todo_category][0]=='\0')) {
+   if ((todo_category != CATEGORY_ALL)
+       && (todo_app_info.category.name[todo_category][0]=='\0')) {
       todo_category=CATEGORY_ALL;
    }
 
+   /* Create basic GUI with left and right boxes and sliding pane */
    accel_group = gtk_accel_group_new();
    gtk_window_add_accel_group(GTK_WINDOW(gtk_widget_get_toplevel(vbox)),
       accel_group);
+   get_pref(PREF_SHOW_TOOLTIPS, &show_tooltips, NULL);
 
    pane = gtk_hpaned_new();
-   get_pref(PREF_TODO_PANE, &ivalue, &svalue);
+   get_pref(PREF_TODO_PANE, &ivalue, NULL);
    gtk_paned_set_position(GTK_PANED(pane), ivalue);
 
    gtk_box_pack_start(GTK_BOX(hbox), pane, TRUE, TRUE, 5);
@@ -2097,7 +2249,8 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_paned_pack1(GTK_PANED(pane), vbox1, TRUE, FALSE);
    gtk_paned_pack2(GTK_PANED(pane), vbox2, TRUE, FALSE);
 
-   /* Left-hand side of GUI */
+   /* Left side of GUI */
+
    /* Separator */
    separator = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 5);
@@ -2109,22 +2262,22 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    glob_date_label = gtk_label_new(" ");
    gtk_box_pack_start(GTK_BOX(vbox1), glob_date_label, FALSE, FALSE, 0);
    timeout_date(NULL);
-   glob_date_timer_tag = gtk_timeout_add(get_timeout_interval(), timeout_date, NULL);
+   glob_date_timer_tag = gtk_timeout_add(CLOCK_TICK, timeout_sync_up, NULL);
 
    /* Separator */
    separator = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 5);
 
-   /* Category Box */
+   /* Left-side Category box */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox_temp, FALSE, FALSE, 0);
 
-   /* left-hand category menu */
+   /* Left-side Category menu */
    make_category_menu(&category_menu1, todo_cat_menu_item1,
                       sort_l, cb_category, TRUE, TRUE);
    gtk_box_pack_start(GTK_BOX(hbox_temp), category_menu1, TRUE, TRUE, 0);
 
-   /* Scrolled window for todo list */
+   /* Todo list scrolled window */
    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
@@ -2186,7 +2339,8 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
    gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(clist));
 
-   /* The right hand part of the main window follows: */
+   /* Right side of GUI */
+
    hbox_temp = gtk_hbox_new(FALSE, 3);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
@@ -2208,7 +2362,7 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
                       GINT_TO_POINTER(UNDELETE_FLAG));
 
    /* Copy button */
-   CREATE_BUTTON(copy_record_button, _("Copy"), COPY, _("Copy the selected record"), GDK_o, GDK_CONTROL_MASK, "Ctrl+O")
+   CREATE_BUTTON(copy_record_button, _("Copy"), COPY, _("Copy the selected record"), GDK_c, GDK_CONTROL_MASK|GDK_SHIFT_MASK, "Ctrl+Shift+C")
    gtk_signal_connect(GTK_OBJECT(copy_record_button), "clicked",
                       GTK_SIGNAL_FUNC(cb_add_new_record),
                       GINT_TO_POINTER(COPY_FLAG));
@@ -2246,7 +2400,11 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
-   /* right-hand category menu */
+   /* Right-side Category menu */
+   /* Clear GTK option menus before use */
+   for (i=0; i<NUM_TODO_CAT_ITEMS; i++) {
+      todo_cat_menu_item2[i] = NULL;
+   }
    make_category_menu(&category_menu2, todo_cat_menu_item2,
                       sort_l, NULL, FALSE, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox_temp), category_menu2, TRUE, TRUE, 0);
@@ -2259,36 +2417,37 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    todo_completed_checkbox = gtk_check_button_new_with_label(_("Completed"));
    gtk_box_pack_start(GTK_BOX(vbox2), todo_completed_checkbox, FALSE, FALSE, 0);
 
-
    /* Priority radio buttons */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
    label = gtk_label_new(_("Priority:"));
    gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 2);
+   set_tooltip(show_tooltips, glob_tooltips, label, _("Set priority   Alt+#"), NULL);
 
    group = NULL;
    for (i=0; i<NUM_TODO_PRIORITIES; i++) {
       sprintf(str,"%d",i+1);
       radio_button_todo[i] = gtk_radio_button_new_with_label(group, str);
       group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button_todo[i]));
+      gtk_widget_add_accelerator(radio_button_todo[i], "clicked", accel_group,
+                                 GDK_1+i, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
       gtk_box_pack_start(GTK_BOX(hbox_temp),
                          radio_button_todo[i], FALSE, FALSE, 0);
    }
 
-
-   /* "Due date" label */
+   /* "Date Due:" label */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
 
-   /* spacer to line up */ 
+   /* Spacer to line up */ 
    hbox_temp2 = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_temp), hbox_temp2, FALSE, FALSE, 1);
 
    label = gtk_label_new(_("Date Due:"));
    gtk_box_pack_start(GTK_BOX(hbox_temp), label, FALSE, FALSE, 0);
 
-   /* "Due Date" button */
+   /* "Date Due" button */
    due_date_button = gtk_button_new_with_label("");
    gtk_box_pack_start(GTK_BOX(hbox_temp), due_date_button, FALSE, FALSE, 5);
    gtk_signal_connect(GTK_OBJECT(due_date_button), "clicked",
@@ -2300,13 +2459,12 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
                       GTK_SIGNAL_FUNC(cb_check_button_no_due_date), NULL);
    gtk_box_pack_start(GTK_BOX(hbox_temp), todo_no_due_date_checkbox, FALSE, FALSE, 0);
 
-
    note_pane = gtk_vpaned_new();
    get_pref(PREF_TODO_NOTE_PANE, &ivalue, NULL);
    gtk_paned_set_position(GTK_PANED(note_pane), ivalue);
    gtk_box_pack_start(GTK_BOX(vbox2), note_pane, TRUE, TRUE, 0);
 
-   /* Description */
+   /* Description text box */
    hbox_temp = gtk_hbox_new (FALSE, 0);
    gtk_paned_pack1(GTK_PANED(note_pane), hbox_temp, TRUE, FALSE);
 
@@ -2322,13 +2480,12 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_container_add(GTK_CONTAINER(scrolled_window), todo_desc);
    gtk_box_pack_start_defaults(GTK_BOX(hbox_temp), scrolled_window);
 
-
-   /* Note */
+   /* Note text box */
    vbox_temp = gtk_vbox_new(FALSE, 0);
    gtk_paned_pack2(GTK_PANED(note_pane), vbox_temp, TRUE, FALSE);
 
    label = gtk_label_new(_("Note"));
-   /* Center Note label visually */
+   /* Center "Note" label visually */
    gtk_misc_set_alignment(GTK_MISC(label), 0.506, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox_temp), label, FALSE, FALSE, 0);
 
@@ -2357,10 +2514,11 @@ int todo_gui(GtkWidget *vbox, GtkWidget *hbox)
                       GTK_SIGNAL_FUNC(cb_key_pressed_left_side), todo_desc);
 
    gtk_signal_connect(GTK_OBJECT(todo_desc), "key_press_event",
-                      GTK_SIGNAL_FUNC(cb_key_pressed_right_side), clist);
+                      GTK_SIGNAL_FUNC(cb_key_pressed_right_side), NULL);
 
    gtk_signal_connect(GTK_OBJECT(todo_note), "key_press_event",
-                      GTK_SIGNAL_FUNC(cb_key_pressed_right_side), clist);
+                      GTK_SIGNAL_FUNC(cb_key_pressed_right_side),
+                      GINT_TO_POINTER(1));
 
    /**********************************************************************/
 
