@@ -1,4 +1,4 @@
-/* $Id: utils.h,v 1.80 2010-07-04 16:58:41 judd Exp $ */
+/* $Id: utils.h,v 1.87 2011-02-09 22:09:29 rousseau Exp $ */
 
 /*******************************************************************************
  * utils.h
@@ -52,30 +52,31 @@
 #  define EPN "jpilot"
 #endif
 
-#define FPI_STRING      "-//Judd Montgomery//NONSGML "PN" "VERSION"//EN"
+/* PRODID string for ical export */
+#define FPI_STRING "-//Judd Montgomery//NONSGML "PN" "VERSION"//EN"
 
-/*This is how often the clock updates in milliseconds */
+/* This is how often the clock updates in milliseconds */
 #define CLOCK_TICK 1000
 
-#define CATEGORY_ALL 300
-
+/* How clist widget should appear for main apps */
 #define SHADOW GTK_SHADOW_ETCHED_OUT
 
-/*
- *             define the maximum length of a category name
- *             when expressed in Pilot character set (assuming 15
- *             character plus a delimiter) or in host character
- *             set (might be 50 or more if UTF-8).
- *         Note : host length is temporarily kept as 16 until all
- *             consequences are identified. In the meantime category
- *             names may be (hopefully safely) truncated.
- */
+/*  Define the maximum length of a category name
+ *  when expressed in Pilot character set (assuming 15
+ *  character plus a delimiter) or in host character
+ *  set (might be 50 or more if UTF-8).
+ *  Note : host length is temporarily kept as 16 until all
+ *  consequences are identified.  In the meantime category
+ *  names may be (hopefully safely) truncated.  */
 
 #define NUM_CATEGORIES 16 
 #define HOSTCAT_NAME_SZ 16
 #define PILOTCAT_NAME_SZ 16
 
-/*Used to mark the entry in the clist to add a record */
+/* Constant used by J-Pilot to indicate "All" category */
+#define CATEGORY_ALL 300
+
+/* Used to mark the entry in the clist to add a record */
 #define CLIST_NEW_ENTRY_DATA 100
 #define CLIST_ADDING_ENTRY_DATA 101
 #define CLIST_MIN_DATA 199
@@ -83,12 +84,18 @@
 #define DIALOG_SAID_1           454
 #define DIALOG_SAID_PRINT       454
 #define DIALOG_SAID_FOURTH      454
-#define DIALOG_SAID_CURRENT     454
 #define DIALOG_SAID_2           455
 #define DIALOG_SAID_LAST        455
-#define DIALOG_SAID_ALL         455
 #define DIALOG_SAID_3           456
 #define DIALOG_SAID_CANCEL      456
+#define DIALOG_SAID_4           457
+
+/* Repeat event dialog */
+#define DIALOG_SAID_RPT_CURRENT 454
+#define DIALOG_SAID_RPT_FUTURE  455
+#define DIALOG_SAID_RPT_ALL     456
+#define DIALOG_SAID_RPT_CANCEL  457
+
 /* Import dialog */
 #define DIALOG_SAID_IMPORT_YES  455
 #define DIALOG_SAID_IMPORT_ALL  456
@@ -109,6 +116,7 @@
 #define PIXMAP_BOX_CHECKED   103
 #define PIXMAP_FLOAT_CHECK   104
 #define PIXMAP_FLOAT_CHECKED 105
+#define PIXMAP_SDCARD        106
 
 #define SORT_ASCENDING       100
 #define SORT_DESCENDING      101
@@ -132,7 +140,7 @@
 #define EXPORT_TYPE_ICALENDAR   106
 #define EXPORT_TYPE_LDIF        107
 
-/* Dat file types */
+/* DAT file types */
 #define DAT_DATEBOOK_FILE    10
 #define DAT_ADDRESS_FILE     11
 #define DAT_TODO_FILE        12
@@ -141,6 +149,7 @@
 /* Pilot-link 0.12 is broken and missing pi_uid_t */
 typedef recordid_t pi_uid_t;
 
+/* Unique ID used to locate a record, say after a search or GUI update */
 extern unsigned int glob_find_id;
 
 typedef enum {
@@ -207,9 +216,8 @@ typedef struct MemoList_s {
    MyMemo mmemo;
 } MemoList;
 
-/*
- * New OS PIM applications in OS 5.x
- */
+/* New OS PIM applications in OS 5.x */
+
 /* Contacts */
 typedef struct {
    PCRecType rt;
@@ -253,21 +261,20 @@ struct sorted_cats
    int cat_num;
 };
 
-/* 
- * Takes an array of database names and changes the names them to the new 
- * PIM names
- */
+/* utils.c: The subroutines below are all from utils.c */
+
+/* Takes an array of database names and changes the names
+ * to the new PIM names */
 void rename_dbnames(char dbname[][32]);
 
-/*
- * Returns usage string that needs to be freed by the caller
- */
+/* Return usage string that must be freed by the caller */
 void fprint_usage_string(FILE *out);
 
 int cat_compare(const void *v1, const void *v2);
 
 int get_timeout_interval(void);
 
+gint timeout_sync_up(gpointer data);
 gint timeout_date(gpointer data);
 
 int get_pixmaps(GtkWidget *widget,
@@ -279,10 +286,7 @@ int check_hidden_dir(void);
 
 int read_gtkrc_file(void);
 
-int get_home_file_name(char *file, char *full_name, int max_size);
-
-FILE *jp_open_home_file(char *filename, char *mode);
-int jp_close_home_file(FILE *pc_in);
+int get_home_file_name(const char *file, char *full_name, int max_size);
 
 int unpack_db_header(DBHeader *dbh, unsigned char *buffer);
 
@@ -296,8 +300,6 @@ int undelete_pc_record(AppType app_type, void *VP, int flag);
 
 void get_month_info(int month, int day, int year, int *dow, int *ndim);
 
-unsigned int bytes_to_bin(unsigned char *bytes, unsigned int num_bytes);
-
 void free_mem_rec_header(mem_rec_header **mem_rh);
 
 void print_string(char *str, int len);
@@ -308,10 +310,6 @@ int cleanup_pc_files(void);
 
 int setup_sync(unsigned int flags);
 
-/* Found in jpilot.c */
-void cb_sync(GtkWidget *widget, unsigned int flags);
-void get_compile_options(char *string, int len);
-
 /* Returns the number of the button that was pressed */
 int dialog_generic(GtkWindow *main_window,
                    char *title, int type,
@@ -351,6 +349,12 @@ void entry_set_multiline_truncate(GtkEntry *entry, gboolean value);
 
 void clist_clear(GtkCList *clist);
 
+inline void set_tooltip(int show_tooltip, 
+                        GtkTooltips *tooltips,
+                        GtkWidget *widget,
+                        const gchar *tip_text,
+                        const gchar *tip_private);
+
 void clist_select_row(GtkCList *clist, 
                       int       row,
                       int       column);
@@ -361,93 +365,37 @@ int clist_find_id(GtkWidget *clist,
 
 int check_copy_DBs_to_home(void);
 
-int jp_copy_file(char *src, char *dest);
-
-/*search_gui.c */
-void cb_search_gui(GtkWidget *widget, gpointer data);
-
-/*install_gui.c */
-int install_gui(GtkWidget *main_window, int w, int h, int x, int y);
-/*install_gui.c */
-int install_append_line(const char *line);
-
-/*import_gui.c */
-void import_gui(GtkWidget *main_window, GtkWidget *main_pane,
-                char *type_desc[], int type_int[],
-                int (*import_callback)(GtkWidget *parent_window,
-                                       const char *file_path, int type));
-int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
-                      char *text, struct CategoryAppInfo *cai,
-                      char *old_cat_name,
-                      int priv, int suggested_cat_num, int *new_cat_num);
 
 int verify_csv_header(const char *header, int num_fields, const char *file_name);
 
-/* Returns a dat type, or 0 */
-int dat_check_if_dat_file(FILE *in);
-int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppInfo *ai);
-int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *ai);
-int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai);
-int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai);
-
 void free_search_record_list(struct search_record **sr);
 
-
-int datebook_gui(GtkWidget *vbox, GtkWidget *hbox);
-int address_gui(GtkWidget *vbox, GtkWidget *hbox);
-int todo_gui(GtkWidget *vbox, GtkWidget *hbox);
-int memo_gui(GtkWidget *vbox, GtkWidget *hbox);
-
-int datebook_gui_cleanup(void);
-int address_gui_cleanup(void);
-int todo_gui_cleanup(void);
-int memo_gui_cleanup(void);
-
-void datebook_gui_setdate(int year, int month, int day);
-
-/*
- * Copy src string into dest while escaping quotes with double quotes.
+/* Copy src string into dest while escaping quotes with double quotes.
  * dest could be as long as strlen(src)*2.
- * Return value is the number of chars written to dest.
- */
+ * Return value is the number of chars written to dest. */
 int str_to_csv_str(char *dest, char *src);
 
-/*
- * Copy src string into dest while escaping carriage returns,
+/* Copy src string into dest while escaping carriage returns,
  * backslashes, commas and semicolons.  Also do line wrapping.
  * dest could be as long as strlen(src) * 2 + strlen(src) / 30.
- * Return value is the number of chars written to dest.
- */
+ * Return value is the number of chars written to dest. */
 int str_to_ical_str(char *, int, char *);
 
-/*
- * Same as str_to_ical_str() except doesn't escape semicolons.
- */
+/* Same as str_to_ical_str() except doesn't escape semicolons. */
 int str_to_vcard_str(char *, int, char *);
 
-/*
- * Parse the string and replace CR and LFs with spaces
- */
+/* Parse the string and replace CR and LFs with spaces */
 void remove_cr_lfs(char *str);
 
-/*
- * Parse the string and replace CR and LFs with spaces
- * a null is written if len is reached
- */
+/* Parse the string and replace CR and LFs with spaces
+ * a null is written if len is reached */
 void lstrncpy_remove_cr_lfs(char *dest, char *src, int len);
 
-/*
- * Output LDIF format (RFC 2849) to file.
+/* Output LDIF format (RFC 2849) to file.
  * Name is name of item (e.g. "cn")
- * fmt ... is like printf.
- */
+ * fmt ... is like printf. */
 void ldif_out(FILE *f, char *name, char *fmt, ...);
 
-/*
- * Output base64-encoded string to file
- */
-void base64_out(FILE *, unsigned char *);
-
 void cleanup_path(char *path);
 
 int add_days_to_date(struct tm *date, int n);
@@ -479,102 +427,43 @@ int find_next_rpt_event(struct CalendarEvent *cale,
                         struct tm *srch_start_tm,
                         struct tm *next_tm);
 
-/*from jpilot.c */
-void cb_app_button(GtkWidget *widget, gpointer data);
-
-void call_plugin_gui(int number, int unique_id);
-void plugin_gui_refresh(int unique_id);
-
-/*datebook_gui */
-int datebook_refresh(int first, int do_init);
-
-/*address_gui */
-int address_refresh(void);
-int address_cycle_cat(void);
- 
-/*todo_gui */
-int todo_refresh(void);
-int todo_cycle_cat(void);
-
-/*memo_gui */
-int memo_refresh(void);
-int memo_cycle_cat(void);
-
-/* monthview_gui */
-void monthview_gui(struct tm *date);
-
-/* weekview_gui */
-void weekview_gui(struct tm *date_in);
-
-/* dialer.c */
-int dialog_dial(GtkWindow *main_window, char *string, char *ext);
-
 /* These are in utils.c for now */
 /*
  * DB_name should be without filename ext, e.g. MemoDB
- * num is the number of records counted returned.
- */
+ * num is the number of records counted returned. */
 int pdb_file_count_recs(char *DB_name, int *num);
-/*
- * DB_name should be without filename ext, e.g. MemoDB
- * uid_in the the unique ID to remove from the pdb file
- */
+/* DB_name should be without filename ext, e.g. MemoDB
+ * uid_in the the unique ID to remove from the pdb file */
 int pdb_file_delete_record_by_id(char *DB_name, pi_uid_t uid_in);
-/*
- * DB_name should be without filename ext, e.g. MemoDB
+/* DB_name should be without filename ext, e.g. MemoDB
  * uid_in the the unique ID to modify from the pdb file
- * the other parameters are set in the pdb file for this record.
- */
+ * the other parameters are set in the pdb file for this record.  */
 int pdb_file_modify_record(char *DB_name, void *record_in, int size_in,
                            int attr_in, int cat_in, pi_uid_t uid_in);
-/*
- * DB_name should be without filename ext, e.g. MemoDB
+/* DB_name should be without filename ext, e.g. MemoDB
  * uid is unique id in
  * bufp is a copy of the raw record (unpacked) and should be freed by caller
  * sizep is size of bufp returned
  * idxp is the index in the file rec was found
  * attrp is the attributes
- * catp is the category (index)
- */
+ * catp is the category (index) */
 int pdb_file_read_record_by_id(char *DB_name, 
                                pi_uid_t uid,
                                void **bufp, size_t *sizep, int *idxp,
                                int *attrp, int * catp);
-/*
- * DB_name should be without filename ext, e.g. MemoDB
+/* DB_name should be without filename ext, e.g. MemoDB
  * bufp is the packed app info block
- * size_in is the size of bufp
- */
+ * size_in is the size of bufp */
 int pdb_file_write_app_block(char *DB_name, void *bufp, size_t size_in);
 
-/*
- * This copies the database (pdb, or prc) and writes the DBInfo privided
- * since there is no other way to set it in a file.
- */
+/* This copies the database (pdb, or prc) and writes the DBInfo privided
+ * since there is no other way to set it in a file. */
 int pdb_file_write_dbinfo(char *DB_name, struct DBInfo *Pinfo_in);
 
 void append_anni_years(char *desc, int max, struct tm *date,
                        struct Appointment *a, struct CalendarEvent *cale);
 int get_highlighted_today(struct tm *date);
 
-/* category.c */
-/*
- * widget is a widget inside the main window used to get main window handle
- * db_name should be without filename ext, e.g. MemoDB
- * cai is the category app info.  This should be unpacked by the user since
- * category unpack functions are database specific.
- */
-int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
-/*
- * This changes every record with index old_index and changes it to new_index
- * returns the number of record's categories changed.
- */
-int pdb_file_change_indexes(char *DB_name, int old_index, int new_index);
-int pdb_file_swap_indexes(char *DB_name, int old_cat, int new_cat);
-int edit_cats_change_cats_pc3(char *DB_name, int old_cat, int new_cat);
-int edit_cats_swap_cats_pc3(char *DB_name, int old_cat, int new_cat);
-int edit_cats_change_cats_pdb(char *DB_name, int old_cat, int new_cat);
-
 int make_category_menu(GtkWidget **category_menu,
                        GtkWidget **cat_menu_item,
                        struct sorted_cats *sort_l,
@@ -583,21 +472,106 @@ int make_category_menu(GtkWidget **category_menu,
                        int add_an_all_item,
                        int add_edit_cat_item);
 
+int jp_copy_file(char *src, char *dest);
+FILE *jp_open_home_file(char *filename, char *mode);
+int jp_close_home_file(FILE *pc_in);
+
+/* Routines used for i18n string manipulation */
 void multibyte_safe_strncpy(char *dst, char *src, size_t len);
 char *multibyte_safe_memccpy(char *dst, const char *src, int c, size_t len);
 
-/*************************************
- * convert char code 
- *************************************/
-/* host character set to Palm character set */
+/* host character set of J-Pilot (j) to Palm character set (p) */
 void charset_j2p(char *buf, int max_len, long char_set);
 
-/* Palm character set to host character set */
+/* Palm character set (p) to host character set of J-Pilot (j) */
 void charset_p2j(char *buf, int max_len, int char_set);
 char *charset_p2newj(const char *buf, int max_len, int char_set);
 
+/* Versions of character conversion routines for plugins */
 void jp_charset_j2p(char *buf, int max_len);
 void jp_charset_p2j(char *buf, int max_len);
 char* jp_charset_p2newj(const char *buf, int max_len);
 size_t jp_strftime(char *s, size_t max, const char *format, const struct tm *tm);
+
+/******************************************************************************/
+/* search_gui.c */
+void cb_search_gui(GtkWidget *widget, gpointer data);
+
+/* install_gui.c */
+int install_gui(GtkWidget *main_window, int w, int h, int x, int y);
+int install_append_line(const char *line);
+
+/* import_gui.c */
+void import_gui(GtkWidget *main_window, GtkWidget *main_pane,
+                char *type_desc[], int type_int[],
+                int (*import_callback)(GtkWidget *parent_window,
+                                       const char *file_path, int type));
+int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
+                      char *text, struct CategoryAppInfo *cai,
+                      char *old_cat_name,
+                      int priv, int suggested_cat_num, int *new_cat_num);
+
+/* dat.c */
+/* Returns a dat type, or 0 */
+int dat_check_if_dat_file(FILE *in);
+int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppInfo *ai);
+int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *ai);
+int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai);
+int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai);
+
+/* jpilot.c */
+void cb_app_button(GtkWidget *widget, gpointer data);
+void get_compile_options(char *string, int len);
+
+void call_plugin_gui(int number, int unique_id);
+void plugin_gui_refresh(int unique_id);
+
+/* datebook_gui.c */
+int datebook_gui(GtkWidget *vbox, GtkWidget *hbox);
+int datebook_gui_cleanup(void);
+int datebook_refresh(int first, int do_init);
+void datebook_gui_setdate(int year, int month, int day);
+
+/* address_gui.c */
+int address_gui(GtkWidget *vbox, GtkWidget *hbox);
+int address_gui_cleanup(void);
+int address_refresh(void);
+int address_cycle_cat(void);
+ 
+/* todo_gui.c */
+int todo_gui(GtkWidget *vbox, GtkWidget *hbox);
+int todo_gui_cleanup(void);
+int todo_refresh(void);
+int todo_cycle_cat(void);
+
+/* memo_gui.c */
+int memo_gui(GtkWidget *vbox, GtkWidget *hbox);
+int memo_gui_cleanup(void);
+int memo_refresh(void);
+int memo_cycle_cat(void);
+
+/* monthview_gui.c */
+void monthview_gui(struct tm *date);
+
+/* weekview_gui.c */
+void weekview_gui(struct tm *date_in);
+
+/* dialer.c */
+int dialog_dial(GtkWindow *main_window, char *string, char *ext);
+
+/* category.c */
+/*
+ * widget is a widget inside the main window used to get main window handle
+ * db_name should be without filename ext, e.g. MemoDB
+ * cai is the category app info.  This should be unpacked by the user since
+ * category unpack functions are database specific.  */
+int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
+/* This changes every record with index old_index and changes it to new_index
+ * returns the number of record's categories changed.  */
+int pdb_file_change_indexes(char *DB_name, int old_index, int new_index);
+int pdb_file_swap_indexes(char *DB_name, int old_cat, int new_cat);
+int edit_cats_change_cats_pc3(char *DB_name, int old_cat, int new_cat);
+int edit_cats_swap_cats_pc3(char *DB_name, int old_cat, int new_cat);
+int edit_cats_change_cats_pdb(char *DB_name, int old_cat, int new_cat);
+
 #endif
